package experimentalcode.franz;

import de.lmu.ifi.dbs.paros.graph.Path;
import de.lmu.ifi.dbs.utilities.Arrays2;
import experimentalcode.franz.OSMUtils.PATH_ATTRIBUTES;
import experimentalcode.franz.osm.OSMGraph;
import experimentalcode.franz.osm.OSMLink;
import experimentalcode.franz.osm.OSMNode;
import experimentalcode.franz.simplex.PointPanel;
import experimentalcode.franz.simplex.PointPanel.PointSource;
import experimentalcode.franz.simplex.SimplexControl;
//import experimentalcode.thomas.AllRoundTrips;

import java.awt.CardLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.geom.Point2D;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.Vector;
import java.util.concurrent.ExecutionException;
import java.util.logging.Level;
import java.util.logging.LogManager;
import java.util.logging.Logger;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;
import javax.swing.UIManager;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableRowSorter;
import org.jdesktop.swingx.JXMapViewer;
import org.jdesktop.swingx.mapviewer.AbstractTileFactory;
import org.jdesktop.swingx.mapviewer.GeoPosition;
import org.jdesktop.swingx.mapviewer.Waypoint;
import org.jdesktop.swingx.mapviewer.WaypointPainter;
import org.jdesktop.swingx.painter.CompoundPainter;
import org.jdesktop.swingx.painter.Painter;

public class Demo extends javax.swing.JFrame {

    private enum POSITIONS {

        START, END
    };
    private final Logger log = Logger.getLogger(Demo.class.getName());
    // -- Properties
    private String lru_dir;
    private boolean autoLoadGraph;
    // --
    private Result result;
    private Statistics statistics;
    private OSMNode startPos;
    private OSMNode endPos;
    private MouseListener srcDestChooser = null;
    private OSMGraph<OSMNode<OSMLink>, OSMLink<OSMNode>> graph;
    private LoadGraphWorker loadGraphWorker = null;
    private WaypointPainter<JXMapViewer> startEndPainter = new WaypointPainter<JXMapViewer>();
    private GraphPainter graphPainter = new GraphPainter();
    private PathPainter pathPainter = new PathPainter();
    private NodePainter visitedNodesPainter = new NodePainter();
    private AlgorithmWorker calculator = null;
    private LoadGraphAction loadAction = new LoadGraphAction();
    private Map<Integer, ResultEntry> results = new HashMap<Integer, ResultEntry>();
    private StatisticsFrame statisticsFrame = null;
    // Algorithms:
    // name -> Class
    private final Hashtable<String, Class<? extends Algorithm>> algorithmMap =
            new Hashtable<String, Class<? extends Algorithm>>();
    // map result type -> layout name
    private final Map<Class, String> resultToLayoutName; // cardlayout
    private final Map<Class, SimplexControl> resultToSimplexControl;

    public Demo() throws IOException {
        log.info("start");
        initComponents();

        // add methods to algorithms dropdown
        algorithmComboBox.removeAllItems();
        ((AbstractTileFactory) jXMapKit.getMainMap().getTileFactory()).setThreadPoolSize(10);
        jXMapKit.setAddressLocation(new GeoPosition(47.76168565517294, 11.559205055236816));
        refreshPainters();
        setLocationRelativeTo(null);

        // fill the resultTo*- maps
        Map<Class, String> mapToLayoutName = new Hashtable<Class, String>();
        mapToLayoutName.put(Simplex1Result.class, "simplex1d");
        mapToLayoutName.put(Simplex2Result.class, "simplex2d");
        mapToLayoutName.put(Simplex3Result.class, "simplex3d");
        resultToLayoutName = Collections.unmodifiableMap(mapToLayoutName);

        // fill the resultTo*- maps
        Map<Class, SimplexControl> mapToControl = new Hashtable<Class, SimplexControl>();
        mapToControl.put(Simplex1Result.class, simplexControl1D);
        mapToControl.put(Simplex2Result.class, simplexControl2D);
        mapToControl.put(Simplex3Result.class, simplexControl3D);
        resultToSimplexControl = Collections.unmodifiableMap(mapToControl);

        // register Actionlisterner on SimplexControls' paintpanel to link from the
        // paint panel to the resultList
        simplexControl1D.addMouseListener(new SimplexHighlighter(resultTable));
        simplexControl2D.addMouseListener(new SimplexHighlighter(resultTable));
        simplexControl3D.addMouseListener(new SimplexHighlighter(resultTable));

        // save and restore LRU-directories
        SaveHook s = new SaveHook(this);
        s.restore();
        Runtime.getRuntime().addShutdownHook(s);

        // load graph
        if (autoLoadGraph && lru_dir != null) {
            try {
                File lru = new File(lru_dir);
                File nodes = new File(lru, "nodes.txt");
                File ways = new File(lru, "ways.txt");

                log.fine("starting worker");
                loadGraphWorker = new LoadGraphWorker(nodes, ways);
                loadGraphWorker.init();
                loadGraphWorker.addPropertyChangeListener(loadAction);
                loadGraphWorker.execute();
            } catch (Exception e) {
                log.log(Level.SEVERE, "", e);
            }
        }

        resultTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {

            @Override
            public void valueChanged(ListSelectionEvent e) {
                reloadStatisticsData();
                highlightResult();
            }
        });
    }

    /**
     * called as soon as the graph is loaded
     */
    private void graphLoaded() {
        String statusText =
                String.format("Finished loading graph. %d links, %d nodes with degree > 2, %d total nodes.",
                graph.getLinkCount(), graph.getNodes().size(), graph.getNodes().size() + graph.getNodesWithoutLinksCount());
        statusbarLabel.setText(statusText);
        graphPainter.setGraph(graph);
        Collection<OSMNode<OSMLink>> nodes = graph.getNodes();
        if (nodes.size() > 0) {
            jXMapKit.setCenterPosition(nodes.iterator().next().getGeoPosition());
        }
        jXMapKit.repaint();

//        setStartPos(graph.getNode(283588275));
//        setEndPos(graph.getNode(287610217));
        setStartPos(graph.getNode(443813279));
        setEndPos(graph.getNode(410818));
    }

    private void showStatisticsFrame() {
        if (statisticsFrame == null || !statisticsFrame.isVisible()) {
            statisticsFrame = new StatisticsFrame();
            statisticsFrame.setLocationRelativeTo(null);
        }
        statisticsFrame.setVisible(true);
        reloadStatisticsData();
    }

    private void reloadStatisticsData() {
        if (results.size() <= 0 || statisticsFrame == null || !statisticsFrame.isVisible()) {
            return;
        }

        statisticsFrame.clear();
        statisticsFrame.setData(statistics);

        if (resultTable.getSelectedRowCount() > 0) {
            for (int rowId : resultTable.getSelectedRows()) {
                Integer resultId = (Integer) resultTable.getValueAt(rowId, 0);
                ResultEntry entry = results.get(resultId);
                if (entry != null) {
                    Map<PATH_ATTRIBUTES, String> map = statistics.getPath(entry.getPath());
                    statisticsFrame.addPathData(map, resultId);
                }
            }
        }
    }

    private void reloadAlgorithmComboBox() {
        algorithmBoxModel.removeAllElements();
        List<String> keyList = new ArrayList<String>();
        keyList.addAll(algorithmMap.keySet());
        Collections.sort(keyList);
        for (String name : keyList) {
            algorithmBoxModel.addElement(name);
        }
        algorithmComboBox.setSelectedIndex(0);
    }

    public void addAlgorithm(Class<? extends Algorithm> aClass) {
        try {
            Algorithm instance = aClass.newInstance();
            String name = instance.getName();
            algorithmMap.put(name, aClass);

            // update GUI
            reloadAlgorithmComboBox();
            repaint();
        } catch (InstantiationException ex) {
            log.log(Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            log.log(Level.SEVERE, null, ex);
        }
    }

    private void chooseLocation(POSITIONS p) {
        JXMapViewer map = jXMapKit.getMainMap();
        if (srcDestChooser != null) {
            map.removeMouseListener(srcDestChooser);
        }

        srcDestChooser = new MousePointChooser(p);
        map.addMouseListener(srcDestChooser);
    }

    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        attributeListModel = new javax.swing.DefaultListModel();
        algorithmBoxModel = new javax.swing.DefaultComboBoxModel();
        resultTableModel = new NoEditTableModel();
        javax.swing.JSplitPane horizontalSplit = new javax.swing.JSplitPane();
        verticalSplitPane = new javax.swing.JSplitPane();
        javax.swing.JPanel topPanel = new javax.swing.JPanel();
        javax.swing.JPanel srcDstPanel = new javax.swing.JPanel();
        javax.swing.JLabel sourceLabel = new javax.swing.JLabel();
        javax.swing.JLabel dstLabel = new javax.swing.JLabel();
        srcTextField = new javax.swing.JTextField();
        dstTextField = new javax.swing.JTextField();
        javax.swing.JButton setSrcButton = new javax.swing.JButton();
        javax.swing.JButton setDstButton = new javax.swing.JButton();
        javax.swing.JButton seekSrcButton = new javax.swing.JButton();
        javax.swing.JButton seekDstButton = new javax.swing.JButton();
        javax.swing.JPanel algorithmPanel = new javax.swing.JPanel();
        searchButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();
        busyLabel = new org.jdesktop.swingx.JXBusyLabel();
        algorithmComboBox = new javax.swing.JComboBox();
        javax.swing.JScrollPane attribScrollPane = new javax.swing.JScrollPane();
        attributeList = new javax.swing.JList();
        javax.swing.JScrollPane resultTableScrollPane = new javax.swing.JScrollPane();
        resultTable = new javax.swing.JTable();
        javax.swing.JButton showStatisticsButton = new javax.swing.JButton();
        simplexContainer = new javax.swing.JPanel();
        simplexControl1D = new experimentalcode.franz.simplex.SimplexControl1D();
        simplexControl2D = new experimentalcode.franz.simplex.SimplexControl2D();
        simplexControl3D = new experimentalcode.franz.simplex.SimplexControl3D();
        javax.swing.JPanel rightPanel = new javax.swing.JPanel();
        jXMapKit = new org.jdesktop.swingx.JXMapKit();
        javax.swing.JPanel statusBar = new javax.swing.JPanel();
        statusbarLabel = new javax.swing.JLabel();
        javax.swing.JMenuBar menuBar = new javax.swing.JMenuBar();
        javax.swing.JMenu fileMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem loadGraphItem = new javax.swing.JMenuItem();
        autoloadMenuItem = new javax.swing.JCheckBoxMenuItem();
        paintGraphMenuItem = new javax.swing.JCheckBoxMenuItem();
        visitedNodesItem = new javax.swing.JCheckBoxMenuItem();
        javax.swing.JMenuItem exitItem = new javax.swing.JMenuItem();
        javax.swing.JMenu aboutMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem aboutMenuItem = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("PAROS");
        setMinimumSize(new java.awt.Dimension(800, 600));

        horizontalSplit.setDividerLocation(275);
        horizontalSplit.setDividerSize(7);

        verticalSplitPane.setDividerLocation(500);
        verticalSplitPane.setDividerSize(7);
        verticalSplitPane.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        verticalSplitPane.setResizeWeight(0.5);

        topPanel.setLayout(new java.awt.GridBagLayout());

        srcDstPanel.setBorder(javax.swing.BorderFactory.createEmptyBorder(5, 5, 5, 5));
        srcDstPanel.setLayout(new java.awt.GridBagLayout());

        sourceLabel.setText("Source:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(1, 1, 1, 1);
        srcDstPanel.add(sourceLabel, gridBagConstraints);

        dstLabel.setText("Destination:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(1, 1, 1, 1);
        srcDstPanel.add(dstLabel, gridBagConstraints);

        srcTextField.setEditable(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(1, 1, 1, 1);
        srcDstPanel.add(srcTextField, gridBagConstraints);

        dstTextField.setEditable(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(1, 1, 1, 1);
        srcDstPanel.add(dstTextField, gridBagConstraints);

        setSrcButton.setText("Set");
        setSrcButton.setToolTipText("set start via mouse click");
        setSrcButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                setSrcButtonActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(1, 1, 1, 1);
        srcDstPanel.add(setSrcButton, gridBagConstraints);

        setDstButton.setText("Set");
        setDstButton.setToolTipText("set destination  via mouse click");
        setDstButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                setDstButtonActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.insets = new java.awt.Insets(1, 1, 1, 1);
        srcDstPanel.add(setDstButton, gridBagConstraints);

        seekSrcButton.setText("Seek");
        seekSrcButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                seekSrcButtonActionPerformed(evt);
            }
        });
        srcDstPanel.add(seekSrcButton, new java.awt.GridBagConstraints());

        seekDstButton.setText("Seek");
        seekDstButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                seekDstButtonActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 1;
        srcDstPanel.add(seekDstButton, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        topPanel.add(srcDstPanel, gridBagConstraints);

        algorithmPanel.setBorder(javax.swing.BorderFactory.createEmptyBorder(5, 5, 5, 5));
        algorithmPanel.setLayout(new java.awt.GridBagLayout());

        searchButton.setText("Search");
        searchButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                searchButtonActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = new java.awt.Insets(1, 0, 1, 2);
        algorithmPanel.add(searchButton, gridBagConstraints);

        cancelButton.setText("Cancel");
        cancelButton.setEnabled(false);
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = new java.awt.Insets(1, 2, 1, 2);
        algorithmPanel.add(cancelButton, gridBagConstraints);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(1, 2, 1, 2);
        algorithmPanel.add(busyLabel, gridBagConstraints);

        algorithmComboBox.setModel(algorithmBoxModel);
        algorithmComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                algorithmComboBoxActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(1, 0, 1, 0);
        algorithmPanel.add(algorithmComboBox, gridBagConstraints);

        attribScrollPane.setMinimumSize(new java.awt.Dimension(100, 100));

        attributeList.setModel(attributeListModel);
        attributeList.setVisibleRowCount(5);
        attribScrollPane.setViewportView(attributeList);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(1, 0, 0, 0);
        algorithmPanel.add(attribScrollPane, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        topPanel.add(algorithmPanel, gridBagConstraints);

        resultTableScrollPane.setPreferredSize(new java.awt.Dimension(250, 150));

        resultTable.setAutoCreateRowSorter(true);
        resultTable.setModel(resultTableModel);
        resultTable.setSelectionMode(javax.swing.ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        resultTableScrollPane.setViewportView(resultTable);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 4, 0, 4);
        topPanel.add(resultTableScrollPane, gridBagConstraints);

        showStatisticsButton.setText("show Statistics");
        showStatisticsButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                showStatisticsButtonActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 4, 0, 4);
        topPanel.add(showStatisticsButton, gridBagConstraints);

        verticalSplitPane.setTopComponent(topPanel);

        simplexContainer.setBorder(javax.swing.BorderFactory.createEmptyBorder(4, 4, 4, 4));
        simplexContainer.setLayout(new java.awt.CardLayout());
        simplexContainer.add(simplexControl1D, "simplex1d");
        simplexContainer.add(simplexControl2D, "simplex2d");
        simplexContainer.add(simplexControl3D, "simplex3d");

        verticalSplitPane.setBottomComponent(simplexContainer);

        horizontalSplit.setLeftComponent(verticalSplitPane);

        rightPanel.setLayout(new java.awt.BorderLayout());

        jXMapKit.setAddressLocationShown(false);
        jXMapKit.setDefaultProvider(org.jdesktop.swingx.JXMapKit.DefaultProviders.OpenStreetMaps);
        jXMapKit.setZoom(2);
        rightPanel.add(jXMapKit, java.awt.BorderLayout.CENTER);

        horizontalSplit.setRightComponent(rightPanel);

        getContentPane().add(horizontalSplit, java.awt.BorderLayout.CENTER);

        statusBar.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        statusBar.setLayout(new java.awt.GridBagLayout());

        statusbarLabel.setText(" ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        statusBar.add(statusbarLabel, gridBagConstraints);

        getContentPane().add(statusBar, java.awt.BorderLayout.SOUTH);

        fileMenu.setMnemonic('f');
        fileMenu.setText("File");

        loadGraphItem.addActionListener(loadAction);
        loadGraphItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, java.awt.event.InputEvent.CTRL_MASK));
        loadGraphItem.setText("Load Graph");
        fileMenu.add(loadGraphItem);

        autoloadMenuItem.setSelected(true);
        autoloadMenuItem.setText("autoload graph next time");
        autoloadMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                autoloadMenuItemActionPerformed(evt);
            }
        });
        fileMenu.add(autoloadMenuItem);

        paintGraphMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_P, java.awt.event.InputEvent.CTRL_MASK));
        paintGraphMenuItem.setSelected(true);
        paintGraphMenuItem.setText("paint Graph");
        paintGraphMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                paintGraphMenuItemActionPerformed(evt);
            }
        });
        fileMenu.add(paintGraphMenuItem);

        visitedNodesItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_V, java.awt.event.InputEvent.CTRL_MASK));
        visitedNodesItem.setText("paint visited nodes");
        visitedNodesItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                visitedNodesItemActionPerformed(evt);
            }
        });
        fileMenu.add(visitedNodesItem);

        exitItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F4, java.awt.event.InputEvent.ALT_MASK));
        exitItem.setMnemonic('x');
        exitItem.setText("Exit");
        exitItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitItemActionPerformed(evt);
            }
        });
        fileMenu.add(exitItem);

        menuBar.add(fileMenu);

        aboutMenu.setMnemonic('a');
        aboutMenu.setText("About");

        aboutMenuItem.setText("About");
        aboutMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                aboutMenuItemActionPerformed(evt);
            }
        });
        aboutMenu.add(aboutMenuItem);

        menuBar.add(aboutMenu);

        setJMenuBar(menuBar);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void exitItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitItemActionPerformed
        System.exit(0);
    }//GEN-LAST:event_exitItemActionPerformed

    private void setSrcButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_setSrcButtonActionPerformed
        chooseLocation(POSITIONS.START);
    }//GEN-LAST:event_setSrcButtonActionPerformed

    private void setDstButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_setDstButtonActionPerformed
        chooseLocation(POSITIONS.END);
    }//GEN-LAST:event_setDstButtonActionPerformed

    /**
     * user selected something in the algorithm list
     */
    private void algorithmComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_algorithmComboBoxActionPerformed
        attributeList.clearSelection();
        attributeListModel.clear();

        Object selected = algorithmComboBox.getSelectedItem();
        if (selected != null) {
            try {
                Class<Algorithm> algoClass = (Class<Algorithm>) algorithmMap.get(selected.toString());
                Algorithm algorithm = algoClass.newInstance();
                // set selection model (1-many) according to the number of allowed attributes
                if (algorithm.getNumAllowedAttrbutes() == 1) {
                    attributeList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
                } else {
                    attributeList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
                }
                for (Object o : algorithm.getAttributes()) { // actually this should be a string list
                    attributeListModel.addElement(o.toString());
                }
                if (algorithm.getAttributes().size() > 0) {
                    attributeList.setSelectedIndex(0);
                }
            } catch (InstantiationException ex) {
                log.log(Level.SEVERE, null, ex);
            } catch (IllegalAccessException ex) {
                log.log(Level.SEVERE, null, ex);
            }
        }
    }//GEN-LAST:event_algorithmComboBoxActionPerformed

    private void searchButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_searchButtonActionPerformed
        startAndRunAlgorithm();
    }//GEN-LAST:event_searchButtonActionPerformed

    private void paintGraphMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_paintGraphMenuItemActionPerformed
        refreshPainters();
    }//GEN-LAST:event_paintGraphMenuItemActionPerformed

    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonActionPerformed
        if (calculator != null) {
            calculator.cancel(true);
        }
    }//GEN-LAST:event_cancelButtonActionPerformed

    private void autoloadMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_autoloadMenuItemActionPerformed
        autoLoadGraph = autoloadMenuItem.isSelected();
    }//GEN-LAST:event_autoloadMenuItemActionPerformed

    private void showStatisticsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_showStatisticsButtonActionPerformed
        showStatisticsFrame();
    }//GEN-LAST:event_showStatisticsButtonActionPerformed

    private void seekSrcButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_seekSrcButtonActionPerformed
        openSeekWindow(POSITIONS.START);
    }//GEN-LAST:event_seekSrcButtonActionPerformed

    private void seekDstButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_seekDstButtonActionPerformed
        openSeekWindow(POSITIONS.END);
    }//GEN-LAST:event_seekDstButtonActionPerformed

    private void visitedNodesItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_visitedNodesItemActionPerformed
        showHideVisitedNodes();
    }//GEN-LAST:event_visitedNodesItemActionPerformed

    private void aboutMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_aboutMenuItemActionPerformed
        JFrame f = new AboutFrame();
        f.setVisible(true);
    }//GEN-LAST:event_aboutMenuItemActionPerformed

    private void openSeekWindow(POSITIONS pos) {
        final SeekPositionFrame spf = new SeekPositionFrame(this.graph);
        spf.setVisible(true);
        spf.addWindowListener(new WindowPointChooser(pos));
    }

    private void refreshPainters() {
        JXMapViewer map = jXMapKit.getMainMap();
        List<Painter> list = new ArrayList<Painter>();
        if (paintGraphMenuItem.isSelected()) {
            list.add(graphPainter);
        }
        list.add(visitedNodesPainter);
        list.add(pathPainter);
        list.add(startEndPainter);
        map.setOverlayPainter(new CompoundPainter(list.toArray(new Painter[]{})));
    }

    private void highlightResult() {
        int[] rowIDs = resultTable.getSelectedRows();
        if (rowIDs.length == 0 || results.size() == 0 || resultTable.getRowCount() == 0) {
            return;
        }

        List<ResultEntry> items = new ArrayList<ResultEntry>();
        for (int rowID : rowIDs) {
            rowID = resultTable.convertRowIndexToModel(rowID);
            Integer id = (Integer) resultTableModel.getValueAt(rowID, 0);
            if (results.containsKey(id)) {
                items.add(results.get(id));
            }
        }

        // okay, this call IS ugly
        Class resultClass = results.values().iterator().next().getResult().getClass();
        SimplexControl simplexControl = resultToSimplexControl.get(resultClass);
        assert simplexControl != null;
        List<PointSource> list = new ArrayList<PointSource>();
        if (items.size() == 0) {
            pathPainter.clear();
        } else {
            List<Path<?, ? extends OSMNode>> pathList = new ArrayList<Path<?, ? extends OSMNode>>();
            for (ResultEntry resultEntry : items) {
                pathList.add(resultEntry.getPath());
                list.add(resultEntry);
                log.info("highlighted: " + resultEntry.getPath().toString());
            }
            pathPainter.setPath(pathList);
        }
        simplexControl.setHighlight(list);
        jXMapKit.repaint();
    }

    private void startAndRunAlgorithm() {
        resultTableModel.setRowCount(0);
        simplexControl1D.setPoints(Collections.EMPTY_LIST);
        simplexControl2D.setPoints(Collections.EMPTY_LIST);
        simplexControl3D.setPoints(Collections.EMPTY_LIST);
        pathPainter.setPath(Collections.EMPTY_LIST);
        visitedNodesPainter.setNodes(Collections.EMPTY_LIST);
        repaint();

        if (graph == null) {
            JOptionPane.showMessageDialog(this, "Please load graph first.");
            return;
        }
        if (startPos == null || endPos == null) {
            JOptionPane.showMessageDialog(this, "Both, start and endpoint must be set.");
            return;
        }

        // cancel possibly running algorithm
        if (calculator != null && !calculator.isDone()) {
            resultTableModel.setRowCount(0);
            calculator.cancel(true);
        }

        busyLabel.setBusy(true);
        try {
            calculator = new AlgorithmWorker(initAlgorithm(), this);
            cancelButton.setEnabled(true);
            calculator.execute();
        } catch (Exception ex) {
            JOptionPane.showMessageDialog(this, "Error: " + ex.getMessage());
            log.log(Level.SEVERE, null, ex);
            busyLabel.setBusy(false);
        }
    }

    private void processResult(Algorithm algorithm) {
        result = algorithm.getResult();
        statistics = algorithm.getStatistics();
        if (result == null) {
            return;
        }

        // Set the correct card
        CardLayout layout = (CardLayout) simplexContainer.getLayout();
        layout.show(simplexContainer, resultToLayoutName.get(result.getClass()));

        // reset all simplex controls
        simplexControl1D.setPoints(Collections.EMPTY_LIST);
        simplexControl2D.setPoints(Collections.EMPTY_LIST);
        simplexControl3D.setPoints(Collections.EMPTY_LIST);

        // clear list
        resultTableModel.setRowCount(0);
        results.clear();

        Map<Path, double[]> resultsMap = result.getResults();
        // Set column names
        Vector colNames = new Vector(result.getUnits());
        colNames.add(0, "id");
        resultTableModel.setColumnIdentifiers(colNames);

        {// set columns sortable
            TableRowSorter sorter = new TableRowSorter(resultTableModel);
            resultTable.setRowSorter(sorter);
            for (int i = 0; i < resultTableModel.getColumnCount(); i++) {
                sorter.setComparator(i, new ResultTableColumnSorter());
            }
        }

        double[] ranges = findMaxPerColumn(resultsMap.values());
        List<PointSource> pointSourceList = new ArrayList<PointSource>();
        int i = 1;
        for (Map.Entry<Path, double[]> entry : resultsMap.entrySet()) {
            double[] absolute = entry.getValue().clone();
            double[] relative = entry.getValue().clone();
            Arrays2.div(relative, ranges);
            Arrays2.replaceNaN(relative, 0); // ranges MAY contain zeros

            int resultDimensionality = algorithm.getSelectedAttributes().size();
            ResultEntry resEntry = new ResultEntry(result, entry.getKey(), relative, absolute, i++, resultDimensionality);
            results.put(resEntry.getId(), resEntry);
            resultTableModel.addRow(resEntry.getVector());
            pointSourceList.add(resEntry);
        }
        SimplexControl simplexControl = resultToSimplexControl.get(result.getClass());
        simplexControl.setAttributNames(algorithm.getSelectedAttributes());
        simplexControl.setPoints(pointSourceList);

        if (resultsMap.size() > 0) {
            resultTable.getSelectionModel().clearSelection();
            resultTable.getSelectionModel().setSelectionInterval(0, 0);
        }
        if (statisticsFrame != null && statisticsFrame.isVisible()) {
            showStatisticsFrame();
        }
        showHideVisitedNodes();
    }

    private void showHideVisitedNodes() {
        visitedNodesPainter.clear();
        if (statistics != null && visitedNodesItem.isSelected()) {
            visitedNodesPainter.setNodes(statistics.getVisitedNodes());
        }
        jXMapKit.repaint();
    }

    private double[] findMaxPerColumn(Collection<double[]> entries) {
        double[] maxima = null;
        for (double[] e : entries) {
            if (maxima == null) {
                maxima = e.clone();
            }
            for (int i = 0; i < e.length; i++) {
                maxima[i] = Math.max(maxima[i], e[i]);
            }
        }
        return maxima;
    }

    private void setStartPos(OSMNode node) {
        if (node == null) {
            return;
        }
        GeoPosition pos = node.getGeoPosition();
        srcTextField.setText(pos.getLatitude() + ", " + pos.getLongitude());
        startPos = node;
        updatePosPainter();
    }

    private void setEndPos(OSMNode node) {
        if (node == null) {
            return;
        }
        GeoPosition pos = node.getGeoPosition();
        dstTextField.setText(pos.getLatitude() + ", " + pos.getLongitude());
        endPos = node;
        updatePosPainter();
    }

    private void updatePosPainter() {
        Set<Waypoint> waypoints = new HashSet<Waypoint>();
        if (startPos != null) {
            waypoints.add(new Waypoint(startPos.getGeoPosition()));
        }
        if (endPos != null) {
            waypoints.add(new Waypoint(endPos.getGeoPosition()));
        }
        startEndPainter.setWaypoints(waypoints);
        jXMapKit.repaint();
    }

    private Algorithm initAlgorithm() throws InstantiationException,
            IllegalAccessException {
        List<String> attributes = new ArrayList<String>();
        for (Object o : attributeList.getSelectedValues()) {
            attributes.add(o.toString());
        }
        String algoName = algorithmComboBox.getSelectedItem().toString();

        Algorithm algorithm = algorithmMap.get(algoName).newInstance();
        algorithm.setGraph(graph);
        algorithm.setAttributes(attributes);
        algorithm.setStartNode(startPos);
        algorithm.setEndNode(endPos);

        return algorithm;
    }

    void setLruDir(String lru) {
        this.lru_dir = lru;
    }

    String getLruDir() {
        return lru_dir;
    }

    boolean isAutoLoad() {
        return autoLoadGraph;
    }

    void setAutoLoad(boolean autoLoad) {
        this.autoLoadGraph = autoLoad;
        autoloadMenuItem.setSelected(autoLoad);
    }

    /**
     * Opens the filechooser for the load graph action.
     * Upon File selection, a new SwingWorker ist started to load the files.
     *
     * Also updates the apinter as soon as the graph is loaded
     */
    class LoadGraphAction implements ActionListener, PropertyChangeListener {

        @Override
        public void actionPerformed(final ActionEvent e) {
            if (loadGraphWorker != null) {
                log.fine("canceling active worker");
                loadGraphWorker.cancel(true);
            }

            // opening the FileChooser MAY take some ugly time which freezes the GUI :-/
            // if this is an issue, but this into an invokeLater()
            JFileChooser chooser = new JFileChooser(lru_dir);
            chooser.setMultiSelectionEnabled(true);

            // Display only directories, nodes.txt and ways.txt in the dropdown
            chooser.setFileFilter(new FileFilter() {

                @Override
                public boolean accept(File f) {
                    return f.isDirectory()
                            || f.getName().equalsIgnoreCase("nodes.txt")
                            || f.getName().equalsIgnoreCase("ways.txt");
                }

                @Override
                public String getDescription() {
                    return "ways.txt & nodes.txt";
                }
            });

            // If the user pressed "okay", try to load the files
            if (JFileChooser.APPROVE_OPTION == chooser.showOpenDialog(getContentPane())) {
                // setting lru dir
                log.fine("saving least recently used dir: " + lru_dir);
                lru_dir = chooser.getSelectedFiles()[0].getParent();
                if (chooser.getSelectedFiles().length != 2) {
                    JOptionPane.showInternalMessageDialog(getContentPane(),
                            "You must load 2 files at once:\n- nodes.txt\n- ways.");
                    return;
                }

                try {
                    log.fine("starting worker");
                    loadGraphWorker = new LoadGraphWorker(chooser.getSelectedFiles());
                    loadGraphWorker.init();
                    loadGraphWorker.addPropertyChangeListener(this);
                    loadGraphWorker.execute();
                } catch (Throwable t) {
                    log.log(Level.SEVERE, "couldn't init graph loader:", t.getMessage());
                    JOptionPane.showInternalMessageDialog(getContentPane(), t.getMessage());
                }
            }
        }

        /**
         * Method that is called as soon as the graphloader task has finished
         * @param evt
         */
        @Override
        public void propertyChange(PropertyChangeEvent evt) {
            if (evt.getNewValue().equals(SwingWorker.StateValue.DONE)) {
                try {
                    if (!loadGraphWorker.isCancelled()) {
                        graph = loadGraphWorker.get();
                        graphLoaded();
                    }
                } catch (InterruptedException ex) {
                    log.log(Level.SEVERE, null, ex);
                } catch (ExecutionException ex) {
                    log.log(Level.SEVERE, null, ex);
                }
            }
        }
    }

    /**
     * Mouseadapter that handles the selection of start and endpoints on the main map
     */
    class MousePointChooser extends MouseAdapter {

        private final POSITIONS targetPos;

        private MousePointChooser(POSITIONS p) {
            this.targetPos = p;
        }

        @Override
        public void mouseClicked(MouseEvent e) {
            GeoPosition pos = jXMapKit.getMainMap().convertPointToGeoPosition(e.getPoint());
            OSMNode node = OSMUtils.getNearestNode(pos, graph);

            if (targetPos == POSITIONS.START) {
                setStartPos(node);
            } else if (targetPos == POSITIONS.END) {
                setEndPos(node);
            }

            // we're done! remove the listener and paint the waypoints on the map
            jXMapKit.getMainMap().removeMouseListener(this);
        }
    }

    /**
     * WindowAdapter that handles the selection of start and endpoints via full
     * text search in an external window
     */
    class WindowPointChooser extends WindowAdapter {

        private final POSITIONS targetPos;

        private WindowPointChooser(POSITIONS p) {
            this.targetPos = p;
        }

        @Override
        public void windowClosed(WindowEvent e) {
            SeekPositionFrame spf = (SeekPositionFrame) e.getWindow();
            spf.removeWindowListener(this);
            OSMNode node = spf.getNode();

            if (targetPos == POSITIONS.START) {
                setStartPos(node);
            } else if (targetPos == POSITIONS.END) {
                setEndPos(node);
            }
        }
    }

    /**
     * swinworker that starts the algorithm and waits for tis termination.
     */
    class AlgorithmWorker extends SwingWorker<Void, Void> {

        private final Logger log = Logger.getLogger(AlgorithmWorker.class.getName());
        private final Component parent;
        private final Algorithm algorithm;

        AlgorithmWorker(Algorithm alg, Component parent) {
            this.algorithm = alg;
            this.parent = parent;
        }

        @Override
        protected Void doInBackground() throws Exception {
            log.info("start algorithm");
            if (algorithm != null) {
                algorithm.run();
            }
            return null;
        }

        @Override
        protected void done() {
            try {
                busyLabel.setBusy(false);
                cancelButton.setEnabled(false);
                if (!isCancelled()) {
                    get();
                    processResult(algorithm);
                }

                calculator = null;
            } catch (InterruptedException ex) {
                log.log(Level.FINE, null, ex);
            } catch (ExecutionException t) {
                log.log(Level.SEVERE, "Algorithm terminated by uncaught exception:", t);
                JOptionPane.showMessageDialog(parent, "Exception in Calculator: " + t.getMessage());
            }
        }
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.DefaultComboBoxModel algorithmBoxModel;
    private javax.swing.JComboBox algorithmComboBox;
    private javax.swing.JList attributeList;
    private javax.swing.DefaultListModel attributeListModel;
    private javax.swing.JCheckBoxMenuItem autoloadMenuItem;
    private org.jdesktop.swingx.JXBusyLabel busyLabel;
    private javax.swing.JButton cancelButton;
    private javax.swing.JTextField dstTextField;
    private org.jdesktop.swingx.JXMapKit jXMapKit;
    private javax.swing.JCheckBoxMenuItem paintGraphMenuItem;
    private javax.swing.JTable resultTable;
    private javax.swing.table.DefaultTableModel resultTableModel;
    private javax.swing.JButton searchButton;
    private javax.swing.JPanel simplexContainer;
    private experimentalcode.franz.simplex.SimplexControl1D simplexControl1D;
    private experimentalcode.franz.simplex.SimplexControl2D simplexControl2D;
    private experimentalcode.franz.simplex.SimplexControl3D simplexControl3D;
    private javax.swing.JTextField srcTextField;
    private javax.swing.JLabel statusbarLabel;
    private javax.swing.JSplitPane verticalSplitPane;
    private javax.swing.JCheckBoxMenuItem visitedNodesItem;
    // End of variables declaration//GEN-END:variables

    public static void main(String args[]) throws Exception {
        LogManager.getLogManager().readConfiguration(Demo.class.getResourceAsStream("./logging.properties"));
        UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());

        java.awt.EventQueue.invokeLater(new Runnable() {

            @Override
            public void run() {
                try {
                    Demo d = new Demo();
                    d.addAlgorithm(OSMDijkstra.class);
                    d.addAlgorithm(OSMSkyline.class);
                    d.addAlgorithm(DemoAlgorithm1D.class);
                    d.addAlgorithm(DemoAlgorithm2D.class);
                    d.addAlgorithm(DemoAlgorithm3D.class);
                    d.setVisible(true);
                } catch (IOException ex) {
                    Logger.getLogger(Demo.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        });
    }
}

class ResultTableColumnSorter implements Comparator {

    @Override
    public int compare(Object o1, Object o2) {
        double d1 = Double.parseDouble(o1.toString().split(" ")[0]);
        double d2 = Double.parseDouble(o2.toString().split(" ")[0]);
        return Double.compare(d1, d2);
    }
}

class ResultEntry implements PointSource {

    private final Result result;
    private final double[] costRelative;
    private final double[] costAbsolute;
    private final int id;
    private final Path path;
    private final int dimensionality;

    ResultEntry(Result result, Path path, double[] costRelative, double[] costAbsolute, int id, int dimensionality) {
        assert Arrays2.findNaN(costRelative) < 0 : "NaN found in costRelative: " + Arrays2.join(costRelative, "|");
        assert Arrays2.findNaN(costAbsolute) < 0 : "NaN found in costAbsolute: " + Arrays2.join(costAbsolute, "|");
        this.result = result;
        this.costRelative = costRelative;
        this.costAbsolute = costAbsolute;
        this.id = id;
        this.path = path;
        this.dimensionality = dimensionality;
    }

    public Vector getVector() {
        Vector vec = new Vector();
        vec.add(id);
        for (int i = 0; i < costRelative.length; i++) {
            String unit = "";
            if (result.getUnits().size() > i) {
                unit = result.getUnits().get(i);
            }
            vec.add(String.format(Locale.US, "%.4f %s", costAbsolute[i], unit));
        }
        return vec;
    }

    public double[] getCost() {
        return costRelative;
    }

    public int getId() {
        return id;
    }

    public Path getPath() {
        return path;
    }

    public Result getResult() {
        return result;
    }

    @Override
    public Point2D getLocation() {
        if (dimensionality == 1) {
            return new Point2D.Double(costRelative[0], 0);
        } else if (dimensionality == 2) {
            return new Point2D.Double(costRelative[0], costRelative[1]);
        } else {
            throw new UnsupportedOperationException("Unsupported cost size = " + dimensionality);
        }
    }

    @Override
    public double[] getCoordinates() {
        return costRelative;
    }

    @Override
    public String toString() {
        String s = id + ": ";
        for (int i = 0; i < costRelative.length; i++) {
            String unit = "";
            if (result.getUnits().size() > i) {
                unit = result.getUnits().get(i);
            }
            s += String.format("%.4f %s | ", costAbsolute[i], unit);
        }
        return s;
    }
}

/**
 * Link the clicks on the simplexcontrols' paintpanels with the selection of the
 * result list
 *
 * @author graf
 */
class SimplexHighlighter extends MouseAdapter {

    private final JTable target;

    SimplexHighlighter(JTable resultList) {
        this.target = resultList;
    }

    @Override
    public void mouseClicked(MouseEvent e) {
        SimplexControl control = (SimplexControl) e.getSource();
        PointPanel panel = control.getPointPanel();
        // convert to click on the panel
        e = SwingUtilities.convertMouseEvent((Component) control, e, control.getPointPanel());

        List<PointSource> pointSources2D = panel.getEpsilonPoints(e.getPoint(), panel.getPointSize() / 2);
        panel.setHighlight(pointSources2D);
        final List<PointSource> pointSources3D = control.getSourceFor(pointSources2D);

        if (pointSources3D.size() > 0 && pointSources3D.get(0) instanceof ResultEntry) {
            int[] resultIDs = new int[pointSources3D.size()];
            for (int i = 0; i < pointSources3D.size(); i++) {
                resultIDs[i] = ((ResultEntry) pointSources3D.get(i)).getId();
            }

            int maxRow = target.getModel().getRowCount();
            target.getSelectionModel().clearSelection();
            for (int rowIndex = 0; rowIndex < maxRow; rowIndex++) {
                int id = (Integer) target.getValueAt(rowIndex, 0);
                if (Arrays2.indexOf(resultIDs, id) >= 0) {
                    target.getSelectionModel().addSelectionInterval(rowIndex, rowIndex);
                }
            }
        }
    }
}

/**
 * TableModel that does not allow editing of cells
 * @author Franz
 */
class NoEditTableModel extends DefaultTableModel {

    @Override
    public boolean isCellEditable(int row, int column) {
        return false;
    }
}
